import { State, Agile, Observer, StateConfigInterface, Collection, StateIngestConfigInterface } from '../internal';
export declare class Computed<ComputedValueType = any> extends State<ComputedValueType> {
    agileInstance: () => Agile;
    computeFunction: () => ComputedValueType;
    deps: Array<Observer>;
    hardCodedDeps: Array<Observer>;
    /**
     * @public
     * Computed - Function that recomputes its value if a dependency changes
     * @param agileInstance - An instance of Agile
     * @param computeFunction - Function for computing value
     * @param config - Config
     */
    constructor(agileInstance: Agile, computeFunction: () => ComputedValueType, config?: ComputedConfigInterface);
    /**
     * @public
     * Recomputes Value of Computed
     * @param config - Config
     */
    recompute(config?: RecomputeConfigInterface): this;
    /**
     * @public
     * Applies new compute Function to Computed
     * @param computeFunction - New Function for computing value
     * @param deps - Hard coded dependencies of Computed Function
     * @param config - Config
     */
    updateComputeFunction(computeFunction: () => ComputedValueType, deps?: Array<SubscribableAgileInstancesType>, config?: UpdateComputeFunctionConfigInterface): this;
    /**
     * @internal
     * Recomputes value and adds missing dependencies to Computed
     */
    compute(config?: ComputeConfigInterface): ComputedValueType;
    patch(): this;
    persist(): this;
    invert(): this;
}
/**
 * @param computedDeps - Hard coded dependencies of Computed Function
 */
export interface ComputedConfigInterface extends StateConfigInterface {
    computedDeps?: Array<SubscribableAgileInstancesType>;
}
/**
 * @param autodetect - If dependencies get autodetected
 */
export interface ComputeConfigInterface {
    autodetect?: boolean;
}
/**
 * @param overwriteDeps - If old hardCoded deps get overwritten
 */
export interface UpdateComputeFunctionConfigInterface extends RecomputeConfigInterface {
    overwriteDeps?: boolean;
}
export interface RecomputeConfigInterface extends StateIngestConfigInterface, ComputeConfigInterface {
}
declare type SubscribableAgileInstancesType = State | Collection | Observer;
export {};
