"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Persistent = void 0;
const internal_1 = require("../internal");
class Persistent {
    constructor(agileInstance, config = {}) {
        this.ready = false;
        this.isPersisted = false;
        this.storageKeys = [];
        this.agileInstance = () => agileInstance;
        this._key = Persistent.placeHolderKey;
        config = internal_1.defineConfig(config, {
            instantiate: true,
            storageKeys: [],
        });
        this.agileInstance().storages.persistentInstances.add(this);
        if (config.instantiate)
            this.instantiatePersistent({
                storageKeys: config.storageKeys,
                key: config.key,
            });
    }
    set key(value) {
        this.setKey(value);
    }
    get key() {
        return this._key;
    }
    setKey(value) {
        this._key = value;
    }
    instantiatePersistent(config = {}) {
        this._key = this.formatKey(config.key) || Persistent.placeHolderKey;
        this.assignStorageKeys(config.storageKeys);
        this.validatePersistent();
    }
    validatePersistent() {
        let isValid = true;
        if (this._key === Persistent.placeHolderKey) {
            internal_1.Agile.logger.error('No valid persist Key found! Please provide a Key or assign one to the parent instance.');
            isValid = false;
        }
        if (!this.defaultStorageKey || this.storageKeys.length <= 0) {
            internal_1.Agile.logger.error('No persist Storage Key found! Please provide at least one Storage Key.');
            isValid = false;
        }
        this.ready = isValid;
        return isValid;
    }
    assignStorageKeys(storageKeys = []) {
        const storages = this.agileInstance().storages;
        if (storageKeys.length <= 0) {
            this.storageKeys = [];
            if (storages.defaultStorage) {
                const key = storages.defaultStorage.key;
                this.defaultStorageKey = key;
                this.storageKeys.push(key);
            }
            return;
        }
        this.storageKeys = storageKeys;
        this.defaultStorageKey = storageKeys[0];
    }
    initialLoading() {
        return __awaiter(this, void 0, void 0, function* () {
            const success = yield this.loadPersistedValue();
            if (this.onLoad)
                this.onLoad(success);
            if (!success)
                yield this.persistValue();
        });
    }
    loadPersistedValue() {
        return __awaiter(this, void 0, void 0, function* () {
            internal_1.Agile.logger.error(`'loadPersistedValue' function isn't Set in Persistent! Be aware that Persistent is no stand alone class!`);
            return false;
        });
    }
    persistValue() {
        return __awaiter(this, void 0, void 0, function* () {
            internal_1.Agile.logger.error(`'persistValue' function isn't Set in Persistent! Be aware that Persistent is no stand alone class!`);
            return false;
        });
    }
    removePersistedValue() {
        return __awaiter(this, void 0, void 0, function* () {
            internal_1.Agile.logger.error(`'removePersistedValue' function isn't Set in Persistent! Be aware that Persistent is no stand alone class!`);
            return false;
        });
    }
    formatKey(key) {
        return key;
    }
}
exports.Persistent = Persistent;
Persistent.placeHolderKey = '__THIS_IS_A_PLACEHOLDER__';
